module Stuka

  module StukaPaths

    RescuableExceptions = [LoadError, NameError]

    def self.processes_definition_path

      config_definition_path = get_config_item("ProcessDefinitionPath")

      if config_definition_path
        return "#{Dir.pwd}/#{config_definition_path}"
      else
        PROCESSES_DEFINITION_PATH
      end

    end

    def self.processes_source_path

      config_source_path = get_config_item("ProcessSourcePath")

      if config_source_path
        return "#{Dir.pwd}/#{config_source_path}"
      else
        PROCESSES_SOURCE_PATH
      end

    end

    def self.actions_source_path

      config_source_path = get_config_item("ActionSourcePath")

      if config_source_path
        return "#{Dir.pwd}/#{config_source_path}"
      else
        ACTIONS_SOURCE_PATH
      end

    end

    def self.relative_processes_definition_path

      config_item = get_config_item("ProcessDefinitionPath")

      if config_item
        return config_item
      else
        RELATIVE_PROCESSES_DEFINITION_PATH
      end

    end

    def self.relative_processes_source_path

      config_item = get_config_item("ProcessSourcePath")

      if config_item
        return config_item
      else
        RELATIVE_PROCESSES_SOURCE_PATH
      end

    end

    def self.relative_actions_source_path

      config_item = get_config_item("ActionSourcePath")

      if config_item
        return config_item
      else
        RELATIVE_ACTIONS_SOURCE_PATH
      end

    end

    def self.get_config_item(name)
      begin
        require_relative "#{Dir.pwd}/stuka_config"
        return StukaConfig.const_get(name)
      rescue *RescuableExceptions
        return false
      end
    end

    #defaults

    PROCESSES_DEFINITION_PATH = "#{Dir.pwd}/process_definition"
    
    PROCESSES_SOURCE_PATH = "#{Dir.pwd}/lib/processes"
    ACTIONS_SOURCE_PATH = "#{Dir.pwd}/lib/actions"

    RELATIVE_PROCESSES_DEFINITION_PATH = "process_definition"
    
    RELATIVE_PROCESSES_SOURCE_PATH = "lib/processes"
    RELATIVE_ACTIONS_SOURCE_PATH = "lib/actions"

    TEST_PROCESSES_DEFINITION_PATH = "#{File.expand_path(File.dirname(__FILE__))}/../../../spec/unit/internal/processes_dsl_test"
    TEST_PROCESSES_INCORRECT_DEFINITION_PATH = "#{File.expand_path(File.dirname(__FILE__))}/../../../spec/unit/internal/processes_dsl_test_incorrect"

    TEST_PROCESSES_SOURCE_PATH = "#{File.expand_path(File.dirname(__FILE__))}/../../../spec/unit/internal/processes_source_test"
    TEST_ACTIONS_SOURCE_PATH = "#{File.expand_path(File.dirname(__FILE__))}/../../../spec/unit/internal/actions_source_test"
    TEMPLATES_PATH = "#{File.expand_path(File.dirname(__FILE__))}/../templates"

  end

end