require 'observer'
require 'stuka/external/dispatcher'
require 'stuka/external/action_finder'

Dir["#{File.dirname(__FILE__)}/process_states/*.rb"].each { |file| require_relative file }

module Stuka::External

  class BusinessProcess

    PROCESS_STATES = {
      inactive: ProcessStates::InactiveState.new,
      started: ProcessStates::StartedState.new,
      finished: ProcessStates::FinishedState.new
    }

    include Observable

    attr_reader :options, :process_id, :start_block, :current_block, :current_action, :current_state, :process_params, :action_params, :dispatcher, :action_finder

    def initialize(process_id: nil, options: {}, dispatcher: Dispatcher.new, action_finder: ActionFinder.new)
      @process_id = process_id
      @options = options
      @dispatcher = dispatcher
      @action_finder = action_finder
      change_state :inactive
    end

    def run(params = {})
      yield(@dispatcher) if block_given?
      set_process_params params
      transition(params, nil)
    end

    def transition(params, transition)
      set_action_params params
      notify
      current_state.handle_transition(self, transition)
    end

    def can_transition_to?(transition)
      blocks.has_key?(current_block) && blocks[current_block].has_key?(transition)
    end

    def blocks
      self.class::Blocks
    end

    private

    def next_block(transition)
      raise "transition not defined" unless can_transition_to?(transition)
      return blocks[current_block][transition]
    end

    def method_missing(name, args)
      change_state :finished
      if dispatcher.can_end_with? name
        dispatcher.call(name.to_sym, args)
      else
        super
      end
    end

    def set_current_block(new_block)
      @current_block = new_block
      @current_action = blocks[new_block][:action]
    end

    def change_state(new_state)
      @current_state = PROCESS_STATES[new_state]
    end

    def set_process_params(params)
      @process_params = params
    end

    def set_action_params(params)
      @action_params = params
    end

    def notify
      changed true
      notify_observers(self)
    end

  end

end
