require_relative "domain"

module Stuka

  class Block < Domain

    attr_accessor :defining_action, :referenced_action_name, :definition_file

    # must not be nil
    ATTRS = ["name", "endings"]

    # must not be empty
    REQUIRED_ATTRS = ["name", "endings"]

    def initialize(name, endings = {})
      super()
      @name = name
      @endings = endings
    end

    def reference_action(namespaced_name)
      raise "already defining an action for block" if @defining_action
      @referenced_action_name = namespaced_name
    end

    def action(action_namespaced_name, action_description = nil)
      raise "action already referenced for block" if referenced_action_name
      action_namespace, action_name = action_namespaced_name.split("::")
      action_description = action_name unless action_description
      @defining_action = Action.new(action_name, action_namespace, action_description)
      @defining_action.definition_file = definition_file
      @defining_action.endings(@endings.keys) if @endings
    end

    def endings(endings_hash = nil)
      if endings_hash
        @endings = endings_hash
        defining_action.endings(endings_hash.keys) if defining_action
      else
        @endings
      end
    end

    def action_namespaced_name
      raise "no action in block" unless (defining_action || referenced_action_name)
      if @defining_action
        defining_action.full_name
      else
        referenced_action_name
      end
    end

  end

end