require_relative "domain"
require_relative "block"

module Stuka

  class Process < Domain

    attr_reader :blocks
    attr_accessor :definition_file

    # must not be nil
    ATTRS = ["name", "description", "start_block"]

    # must not be empty
    REQUIRED_ATTRS = ["name", "start_block"]

    def initialize(name, description = nil, start_block = nil)
      super()
      @name = name
      @description = description
      @start_block = start_block
      @blocks = []
      @definition_file = ""
    end

    def block(block_name, &block)
      raise "block method requires a block" unless block_given?
      process_block = Block.new(block_name)
      process_block.definition_file = definition_file
      process_block.instance_eval(&block)
      @blocks << process_block
    end

    def actions
      all_actions = blocks.select { |block| block.defining_action }.map(&:defining_action)
      filtered_actions = []
      all_actions.each do |acc|
        next if filtered_actions.map(&:name).include? acc.name
        filtered_actions << acc
      end
      filtered_actions
    end

  end

end