require 'stuka/internal/domain_builders/action_domain_builder'
require 'stuka/internal/source_builders/source_builder'
require 'stuka/monkeypatch/string_patch'
require 'tempfile'
require 'stuka/config/template_config'

module Stuka

  class ActionBuilder < SourceBuilder

    def self.build(stream)
      
      adb = ActionDomainBuilder.new
      adb.build_domain

      build_files(adb.instances, stream)

    end

    def self.build_files(instances, stream)

      template_file_header = "#{templates_path}/source/action_header.tt"
      template_file = "#{templates_path}/source/action.tt"

      FileUtils.mkdir_p Stuka::StukaPaths.actions_source_path

      instances.each do |acc|
        action_file_path = "#{source_path}/#{acc.namespace}"
        FileUtils.mkdir_p action_file_path

        if File.file?("#{action_file_path}/#{acc.name}.rb")
          stream.puts "action file #{acc.namespace}::#{acc.name} already exists."
          next
        end

        # generate action header
        action_string = ERB.new(File.read(template_file_header)).result(acc.get_binding)

        # generate namespace modules
        modules = Stuka::TemplateConfig.action_module_namespaces
        modules.each_with_index do |mod, index|
          action_string += ("\t" * index)
          action_string += "module #{mod}\n"
        end

        action_class = ERB.new(File.read(template_file)).result(acc.get_binding)

        # generate action class with tabs
        action_class.lines.each do |line|
          action_string += ("\t" * modules.count)
          action_string += line
        end

        # generate module ends
        modules.each_with_index do |mod, index|
          action_string += ("\t" * (modules.count - index - 1))
          action_string += "end\n"
        end

        # generate end file
        out_file = File.new("#{source_path}/#{acc.namespace}/#{acc.name}.rb", "w")
        out_file.puts(action_string)
        out_file.close

      end

    end

    def self.source_path
      Stuka::StukaPaths.actions_source_path
    end

    private

    def self.refresh_action_file(path, action, renderer)
      temp_file = Tempfile.new('foo')
      begin
        File.open(path, 'r') do |file|
          implementation = ""
          is_in_implement_section = false
          file.each_line do |line|
            if is_in_implement_section
              if line.strip.start_with? "## END_IMPLEMENTATION"
                is_in_implement_section = false 
                next
              end
              implementation += line
            else
              is_in_implement_section = true if line.strip.start_with? "## IMPLEMENTATION"
            end
          end
          
          action.implementation = implementation
          temp_file.puts(renderer.result(action.get_binding))

        end
        
        FileUtils.mv(temp_file.path, path)

      ensure
        temp_file.close
        temp_file.unlink
      end
    end

  end

end
