require 'stuka/internal/domain_builders/process_domain_builder'
require 'stuka/internal/validator'
require 'stuka/internal/source_builders/source_builder'
require 'stuka/internal/source_builders/action_builder'
require 'stuka/monkeypatch/string_patch'

module Stuka

  class ProcessBuilder < SourceBuilder

    def self.build
      pdb = ProcessDomainBuilder.new
      pdb.build_domain

      validator = Stuka::Validator.new
      raise validator.message unless validator.process_set_valid? pdb.process_instances

      template_file_header = "#{templates_path}/source/process_header.tt"
      template_file = "#{templates_path}/source/process.tt"

      FileUtils.mkdir_p source_path

      ActionBuilder.build_files(pdb.action_instances, $stdout)

      pdb.process_instances.each do |proc|
        raise validator.message unless validator.process_valid? proc

        # generate action header
        process_string = ERB.new(File.read(template_file_header)).result(proc.get_binding)

        # generate namespace modules
        modules = Stuka::TemplateConfig.process_module_namespaces
        modules.each_with_index do |mod, index|
          process_string += ("\t" * index)
          process_string += "module #{mod}\n"
        end

        process_class = ERB.new(File.read(template_file)).result(proc.get_binding)

        # generate action class with tabs
        process_class.lines.each do |line|
          process_string += ("\t" * modules.count)
          process_string += line
        end

        # generate module ends
        modules.each_with_index do |mod, index|
          process_string += ("\t" * (modules.count - index - 1))
          process_string += "end\n"
        end

        out_file = File.new("#{source_path}/#{proc.name}.rb", "w")
        out_file.puts(process_string)
        out_file.close
      end

    end

    def self.source_path
      Stuka::StukaPaths.processes_source_path
    end

  end

end